#include "rtc.h"

#include <stdlib.h>

// Erstellen einer echtzeituhr unter verwendung des Timers 0
// mit externen Taktquelle eine 32k quartzes.
// Timerwer kann auch fr verzgerrungen und entprellen verwendet werden.
// 

//*********************************************************
// Init Routine zum Timer anwerfen und Teiler einstellen
//

//volatile uint8_t sek1_4;
volatile uint16_t bigMin1_4;

void InitRtc(void)
{
	sbi(ASSR,AS0); //Assyncron mit uhrenquartz
	TCCR0 |= Timer0Teiler; //Teiler 32
	TIMSK |= _BV(TOIE0);
}

//*********************************************************
//Interrupt routine
//viertelsekunden und groe sekunden erhhen
ISR(TIMER0_OVF_vect)
{
	//sek1_4++;
	bigMin1_4 += 256;
}

//Letzte Tage im Monat
const uint8_t letzterTag[2][12] = {{31,28,31,30,31,30,31,31,30,31,30,31},
                                  {31,29,31,30,31,30,31,31,30,31,30,31}};


struct{
		uint8_t sekunde;//aktuelle sekunde
		uint8_t minute;//aktuelle minute
		uint8_t stunde;//aktuelle stunde
		uint8_t tag;//Aktueller tag
		uint8_t monat;//aktueller monat
		uint16_t jahr;//aktuelles jahr
		uint8_t letzteSekunde;//letzte sekunde
		uint8_t schaltjahr;//Schaltjahr ja/nein
	}datum;


void realTime(void){
	
	uint8_t diff;
	uint8_t ak;
	
	ak = (uint8_t)(get_ticks()/256);
	
	diff = datum.letzteSekunde - ak;
	if(diff>4){//wenn eine Sekunde vergangen ist ...
		datum.letzteSekunde = ak - (diff-4);//letzte runde speichern
		if (++datum.sekunde >59)
  		{
    		datum.sekunde = 0;
    		if (++datum.minute > 59)
    			{
      				datum.minute = 0;
      				if (++datum.stunde >23)
      					{
        					datum.stunde = 0;
        					if (++datum.tag > letzterTag[datum.schaltjahr][datum.monat-1])
        						{
          							datum.tag = 1;
          							if (++datum.monat >12)
        							{
            							datum.monat = 1;
            							++datum.jahr;
            							if ((datum.jahr % 4 == 0) && ((datum.jahr % 100 != 0) || (datum.jahr % 400 == 0)))
            							{
              								datum.schaltjahr = 0;
            							}
            							else
            							{
              								datum.schaltjahr = 1;
            							}
          							}
        						}
      						}
    					}
  					}
		
	}
	
	
	
}



//aktuellen wert vom Timer lesen, 16Bit
uint16_t get_ticks( void )			  // read T0 as 16 bit timer
{
	uint16_t val;
  uint8_t tifr, TSREG;
  
	TSREG=SREG;
 	cli();
  	val = UsedTimer + bigMin1_4;
  	tifr = TIFR;				  // read interrupt flags
  	SREG=TSREG;
  	if( (tifr & 1<<TOV0) && !(val & 0x80) ){ // overflow prior reading TCNT0 ?
    	val += 256;				  // then add overflow
  	}
  return val;
}


/**********************************************************************
 * 
 * Verzgerung fr max 250ms Starten
 * 
 **********************************************************************/

void del_ms_start(uint8_t *data){
	(*data) = UsedTimer;
};
/* ********************************************************************
 * 
 * Verzgerung Abfragen fr 250ms
 * 
 * ********************************************************************/

uint8_t del_ms_read(uint8_t *data, uint8_t value){
			
			if((UsedTimer - (*data)) <= value)
			{
			return 0;
			}
			else
			{
			return 1;
			}
}

/**********************************************************************
 * 
 * Verzgerung fr max 65000ms Starten
 * 
 **********************************************************************/

void del_ms_big_start(uint16_t *data){
	(*data) = get_ticks();
}

/* ********************************************************************
 * 
 * Verzgerung Abfragen fr 65000ms
 * 
 * ********************************************************************/

uint8_t del_ms_big_read(uint16_t *data, uint16_t value){
			
			if((get_ticks()-(*data)) <= value)//Abfrage ob zeit Vergangen
			{
				return 0;
			}
			else
			{
				return 1;
			}
}

 uint16_t getMin(){
	 return bigMin1_4;
 }
